@@ -1,4 +1,4 @@
-# $Id: Admin.pm,v 1.20 2000/08/23 04:25:30 eric Exp $
+# $Id: Admin.pm,v 1.22 2000/10/13 06:32:13 eric Exp $
 
 package IMAP::Admin;
 
@@ -10,7 +10,7 @@ use Text::ParseWords qw(quotewords);
 
 use vars qw($VERSION);
 
-$VERSION = '1.3.0';
+$VERSION = '1.3.7';
 
 sub new {
     my $class = shift;
@@ -45,7 +45,9 @@ sub _initialize {
     if (!defined($self->{'Password'})) {
 	croak "$self->{'CLASS'} not initialized properly : Password parameter missing";
     }
-
+    if (!defined($self->{'Separator'})) {
+	$self->{'Separator'} = "."; # set hiearchical separator to a period
+    }
     if (!eval {$self->{'Socket'} = IO::Socket::INET->new(PeerAddr => $self->{'Server'},
 							 PeerPort => $self->{'Port'},
 							 Proto => 'tcp',
@@ -128,6 +130,33 @@ sub create {
     }
 }
 
+sub rename {
+    my $self = shift;
+
+    if (scalar(@_) != 2) {
+	$self->_error("rename", "incorrect number of arguments");
+	return 1;
+    }
+    my $old_name = shift;
+    my $new_name = shift;
+
+    if (!defined($self->{'Socket'})) {
+	$self->_error("delete", "no connection open to", $self->{'Server'});
+	return 1;
+    }
+    my $fh = $self->{'Socket'};
+    print $fh qq{try RENAME "$old_name" "$new_name"\n};
+    my $try = <$fh>;
+    if ($try =~ /^try OK/) {
+	$self->{'Error'} = 'No Errors';
+	return 0;
+    } else {
+	$self->_error("rename", "couldn't rename", $old_name, "to", $new_name,
+		      ":", $try);
+	return 1;
+    }
+}
+
 sub delete {
     my $self = shift;
 
@@ -151,6 +180,41 @@ sub delete {
 	return 1;
     }
 }
+
+sub h_delete {
+    my $self = shift;
+
+    if (scalar(@_) != 1) {
+	$self->_error("h_delete", "incorrect number of arguments");
+	return 1;
+    }
+    my $mailbox = shift;
+    if (!defined($self->{'Socket'})) {
+	$self->_error("h_delete", "no connection open to", $self->{'Server'});
+	return 1;
+    }
+    my $fh = $self->{'Socket'};
+    # first get a list of all sub boxes then nuke them, accumulate errors
+    # then do something intelligent with them (hmmmmm)
+    my $box = join($self->{'Separator'}, $mailbox, "*");
+    my @sub_boxes = $self->list($box);
+    push @sub_boxes, $mailbox;
+    # uncomment following line if you are sanity checking h_delete
+    # print "h_delete: got this list of sub boxes [@sub_boxes]\n";
+    foreach $box (@sub_boxes) {
+	print $fh qq{try DELETE "$box"\n};
+	my $try = <$fh>;
+	if ($try =~ /^try OK/) {
+	    $self->{'Error'} = 'No Errors';
+	} else {
+	    $self->_error("h_delete", "couldn't delete", 
+			  $mailbox, ":", $try);
+	    return 1; # or just return on the first encountered error ?
+	}
+    }
+    return 0;
+}
+
 sub get_quotaroot { # returns an array or undef
     my $self = shift;
     my (@quota, @info);
@@ -264,6 +328,57 @@ sub set_quota {
     }
 }
 
+sub subscribe {
+    my $self = shift;
+
+    if (scalar(@_) != 1) {
+	$self->_error("subscribe", "incorrect number of arguments");
+	return 1;
+    }
+    if (!defined($self->{'Socket'})) {
+	$self->_error("subscribe", "no connection open to", $self->{'Server'});
+	
+	return 1;
+    }
+    my $mailbox = shift;
+    my $fh = $self->{'Socket'};
+    print $fh qq{try SUBSCRIBE "$mailbox"\n};
+    my $try = <$fh>;
+    if ($try !~ /^try OK/) {
+	$self->_error("subscribe", "couldn't suscribe ", $mailbox, ":",
+		      $try);
+	return 1;
+    }
+    $self->{'Error'} = 'No Errors';
+    return 0;
+}
+
+sub unsubscribe {
+    my $self = shift;
+
+    if (scalar(@_) != 1) {
+	$self->_error("unsubscribe", "incorrect number of arguments");
+	return 1;
+    }
+    if (!defined($self->{'Socket'})) {
+	$self->_error("unsubscribe", "no connection open to",
+		      $self->{'Server'});
+	return 1;
+    }
+    my $mailbox = shift;
+    my $fh = $self->{'Socket'};
+    print $fh qq{try UNSUBSCRIBE "$mailbox"\n};
+    my $try = <$fh>;
+    if ($try !~ /^try OK/) {
+	$self->_error("unsubscribe", "couldn't unsuscribe ", $mailbox, ":",
+		      $try);
+	return 1;
+    }
+    $self->{'Error'} = 'No Errors';
+    return 0;
+}
+
+
 sub get_acl { # returns an array or undef
     my $self = shift;
     my (@info, @acl_item, @acl, $item);
@@ -420,9 +535,10 @@ IMAP::Admin - Perl module for basic IMAP server administration
 =head1 SYNOPSIS
 
   use IMAP::Admin;
-  
+    
   $imap = IMAP::Admin->new('Server' => 'name.of.server.com',
 			   'Port' => port# (143 is default),
+			   'Separator' => ".", # default is a period
 			   'Login' => 'login_of_imap_administrator',
 			   'Password' => 'password_of_imap_adminstrator');
 
@@ -432,6 +548,12 @@ IMAP::Admin - Perl module for basic IMAP server administration
   }
   $err = $imap->create("user.bob", "green"); 
   $err = $imap->delete("user.bob");
+  $err = $imap->h_delete("user.bob");
+
+  $err = $imap->subscribe("user.bob");
+  $err = $imap->unsubscribe("user.bob");
+
+  $err = $imap->rename("bboard", "newbboard");
 
   @quota = $imap->get_quotaroot("user.bob");
   @quota = $imap->get_quota("user.bob");
@@ -456,6 +578,7 @@ It's interface should, in theory, work with any RFC compliant IMAP server, but I
 
 Operationally it opens a socket connection to the IMAP server and logs in with the supplied login and password.  You then can call any of the functions to perform their associated operation.
 
+The Separator on the new call is the hiearchical separator used by the imap server.  It is defaulted to a period ("/" might be another popular one).
 
 =head2 MAILBOX FUNCTIONS
 
@@ -465,14 +588,20 @@ create makes new mailboxes.  Cyrus IMAP, for normal mailboxes, has the user. pre
 create returns a 0 on success or a 1 on failure.  An error message is placed in the object->{'Error'} variable on failure. create takes an optional second argument that is the partition to create the mailbox in (I don't know if partition is rfc or not, but it is supported by Cyrus IMAP and Mirapoint).
 
 delete destroys mailboxes.
-delete returns a 0 on success or a 1 on failure.  An error message is placed in the object->{'Error'} variable on failure.
+The action delete takes varies from server to server depending on it's implementation.  On some servers this is a hierarchical delete and on others this will delete only the mailbox specified and only if it has no subfolders that are marked \Noselect.  If you wish to insure a hierarchical delete use the h_delete command as it deletes starting with the subfolders and back up to the specified mailbox.  delete returns a 0 on success or a 1 on failure.  An error message is placed in the object->{'Error'} variable on failure.
+
+h_delete hierarchical delete (I don't believe this is RFC anything)
+deletes a mailbox and all sub-mailboxes/subfolders that belong to it.  It basically gets a subfolder list and does multiple delete calls.  It returns 0 on sucess or a 1 on failure with the error message from delete being put into the object->{'Error'} variable.  Don't forget to set your Separator if it's not a period.
 
 list lists mailboxes.  list accepts wildcard matching
 
+subscribe/unsubscribe does this action on given mailbox.
+
+rename renames a mailbox.  IMAP servers seem to be peculiar about how they implement this, so I wouldn't necessarily expect it to do what you think it should.
 
 =head2 QUOTA FUNCTIONS
 
-NOT RFC2060 commands.  These are supported by Cyrus IMAP and Mirapoint.
+RFC2087 imap extensions.  These are supported by Cyrus IMAP and Mirapoint.
 
 get_quotaroot and get_quota retrieve quota information.  They return an array on success and undef on failure.  In the event of a failure the error is place in the object->{'Error'} variable.  The array has three elements for each item in the quota. 
 $quota[0] <- mailbox name
@@ -487,7 +616,7 @@ To delete a quota do a set_quota($mailbox, "none");
 
 =head2 ACCESS CONTROL FUNCTIONS
 
-NOT RFC2060 commands.  These are supported by Cyrus IMAP and Mirapoint.
+RFC2086 imap extensions.  These are supported by Cyrus IMAP, Mirapoint and probably many others.
 
 get_acl retrieves acl information.  It returns an array on success and under on failure.  In the event of a failure the error is placed in the object->{'Error'} variable. The array contains a pair for each person who has an acl on this mailbox
 $acl[0] user who has acl information
@@ -498,6 +627,17 @@ set_acl set acl information for a single mailbox.  You can specify more the one
 
 delete_acl removes acl information on a single mailbox for the given users.  You can specify more the one users rights to be removed in the same delete_acl call.  It returns a 0 on success or a 1 on failure.  An error message is placed int the object->{'Error'} variable on failure.
 
+standard rights (rfc2086):
+ l - lookup (mailbox is visible to LIST/LSUB commands)
+ r - read (SELECT the mailbox, perform CHECK, FETCH, PARTIAL, SEARCH, and COPY)
+ s - keep seen/unssen information across sessions (STORE SEEN flag)
+ w - write (STORE flags other then SEEN and DELETED)
+ i - insert (perform APPEND and COPY into mailbox)
+ p - post (send mail to submission address for mailbox)
+ c - create (CREATE new sub-mailboxes) (*note* allows for delete of sub mailboxes as well)
+ d - delete (STORE DELETED flag, perform EXPUNGE)
+ a - administer (perform SETACL)
+
 The access control information is from Cyrus IMAP.
   read   = "lrs"
   post   = "lrsp"
@@ -515,7 +655,7 @@ This is licensed under the Artistic license (same as perl).  A copy of the licen
 
 =head1 CVS REVISION
 
-$Id: Admin.pm,v 1.20 2000/08/23 04:25:30 eric Exp $
+$Id: Admin.pm,v 1.22 2000/10/13 06:32:13 eric Exp $
 
 =head1 AUTHOR
 
